//-*-C++-*-
/*
**************************************************************************
                                 description
                             --------------------
    copyright            : (C) 2002 by Luis Carvalho
    email                : lpassos@mail.telepac.pt
**************************************************************************

**************************************************************************
*                                                                        *
*  This program is free software; you can redistribute it and/or modify  *
*  it under the terms of the GNU General Public License as published by  *
*  the Free Software Foundation; either version 2 of the License, or     *
*  (at your option) any later version.                                   *
*                                                                        *
**************************************************************************/

#include "pmlibraryobject.h"

#include <qstring.h>
#include <qvaluelist.h>
#include <qdom.h>
#include <qbuffer.h>
#include <qimage.h>

#include <karchive.h>
#include <kurl.h>
#include <ktar.h>
#include <kfilterdev.h>

#include <stdlib.h>

#include "pmdebug.h"

PMLibraryObject::PMLibraryObject( )
{
   m_previewLoaded = false;
   m_objectsLoaded = false;
   m_preview = NULL;
   m_objects = NULL;
   m_data = NULL;
   m_name = QString::null;
   m_type = QString::null;
   m_description = QString::null;
}

PMLibraryObject::PMLibraryObject( const KURL& u )
{
   m_previewLoaded = false;
   m_objectsLoaded = false;
   m_preview = NULL;
   m_objects = NULL;
   m_data = new KTar( u.path( ), "application/x-gzip" );
   loadLibraryInfo( );
}

PMLibraryObject::~PMLibraryObject( )
{
   delete m_data;
   if( m_previewLoaded )
      delete m_preview;
   if( m_objectsLoaded )
      delete m_objects;
}

void PMLibraryObject::loadLibraryInfo( )
{
   if( m_data )
   {
      m_data->open( IO_ReadOnly );
      // Position in the root of the file
      const KArchiveDirectory* root = m_data->directory( );
      if( !root )
         return;

      // Find the object info
      const KArchiveEntry* entry = root->entry( "objectinfo.xml" );
      if( entry && entry->isFile( ) )
      {
         QBuffer buffer( ( ( KArchiveFile* )entry )->data( ) );
         buffer.open( IO_ReadOnly );

         QDomDocument doc( "OBJECTINFO" );
         doc.setContent( &buffer );

         QDomElement e = doc.documentElement( );
         m_name = e.attribute( "name", "empty" );
         m_type = e.attribute( "type", "empty" );

         QDomNode c = e.firstChild( );
         while( !c.isNull( ) )
         {
            if( c.isElement( ) )
            {
               QDomElement ce = c.toElement( );
               if( ce.tagName( ) == "description" )
               {
                  QDomText te = ce.firstChild( ).toText( );
                  m_description = te.nodeValue( );
               }
               else if( ce.tagName( ) == "keywordlist" )
               {
                  QDomNode ck = e.firstChild( );
                  while( !ck.isNull( ) )
                  {
                     if( ck.isElement( ) )
                        m_keywords.append( ck.toElement( ).attribute( "text", "" ) );
                     ck = ck.nextSibling( );
                  }
               }
            }
            c = c.nextSibling( );
         }
      }
      m_data->close( );
   }
}

QImage* PMLibraryObject::preview( )
{
   if( !m_previewLoaded )
   {
      if( m_data )
      {
         m_data->open( IO_ReadOnly );
         // Position in the root of the file
         const KArchiveDirectory* root = m_data->directory( );
         if( !root )
            return NULL;

         // Find the object info
         const KArchiveEntry* entry = root->entry( "preview.png" );
         if( entry && entry->isFile( ) )
         {
            QBuffer buffer( ( ( KArchiveFile* )entry )->data( ) );
            buffer.open( IO_ReadOnly );
            m_preview = new QImage( buffer.buffer( ) );
            m_previewLoaded = true;
         }
         m_data->close( );
      }
   }
   return m_preview;
}

QByteArray* PMLibraryObject::objects( )
{
   if( !m_objectsLoaded )
   {
      if( m_data )
      {
         m_data->open( IO_ReadOnly );
         // Position in the root of the file
         const KArchiveDirectory* root = m_data->directory( );
         if( !root )
            return NULL;

         // Find the object info
         const KArchiveEntry* entry = root->entry( "objectdata.kpm" );
         if( entry && entry->isFile( ) )
         {
            // Transfer the file contents to a QByteArray.
            // TODO Maybe there is a smarter way of doing this.
            char buf[256];
            int nbytes;
            QIODevice* aux_in = ( ( KArchiveFile* )entry )->device( );

            m_objects = new QByteArray( );
            QBuffer aux_out( *m_objects );

            aux_in->open( IO_ReadOnly );
            aux_out.open( IO_WriteOnly );
            while( !aux_in->atEnd( ) )
            {
               nbytes = aux_in->readBlock( buf, 256 );
               aux_out.writeBlock( buf, nbytes );
            }
            delete aux_in;
            m_objectsLoaded = true;
         }
      }
   }
   return m_objects;
}

void PMLibraryObject::setName( const QString& str )
{
   m_name = str;
}

void PMLibraryObject::setDescription( const QString& str )
{
   m_description = str;
}

void PMLibraryObject::setType( const QString& str )
{
   m_type = str;
}

void PMLibraryObject::setPreview( const QImage& img )
{
   if( m_previewLoaded )
      delete m_preview;

   m_preview = new QImage( img );
   m_previewLoaded = true;
}

void PMLibraryObject::setObjects( const QByteArray& obj )
{
   if( m_objectsLoaded )
      delete m_objects;

   m_objects = new QByteArray( obj );
   m_objects->detach( );
   m_objectsLoaded = true;
}

bool  PMLibraryObject::addKeyword( const QString& str )
{
   if( m_keywords.find( str ) == m_keywords.end( ) )
   {
      m_keywords.append( str );
      return true;
   }
   else
      return false;
}

bool PMLibraryObject::removeKeyword( const QString& str )
{
   return( m_keywords.remove( str ) > 0 );
}

void PMLibraryObject::clearKeywords( )
{
   m_keywords.clear( );
}

bool PMLibraryObject::containsKeyword( const QString& str )
{
   return m_keywords.find( str ) != m_keywords.end( );
}

void PMLibraryObject::save( const QString& fileName )
{
   // create the library file
   m_data = new KTar( fileName, "application/x-gzip" );
   m_data->open( IO_WriteOnly );
   // save object info
   saveLibraryInfo( );
   // save preview
   savePreview( );
   // save library objects
   saveObjects( );
}

void PMLibraryObject::saveLibraryInfo( )
{
   // Pre-condition for the execution of this method.
   if( !m_data || !m_data->isOpened( ) )
   {
      kdError( PMArea ) << "Trying to save to an unopened data file." << endl;
      exit( 1 );
   }

   QBuffer buf;

   // Document type
   QDomDocument doc( "OBJECTINFO" );
   
   // The root element for the document has two attributes: name and type
   QDomElement root = doc.createElement( "object" );
   doc.appendChild( root );
   root.setAttribute( "name", m_name );
   root.setAttribute( "type", m_type );

   // Inside the root element we add the description node.
   QDomElement description = doc.createElement( "description" );
   root.appendChild( description );
   description.appendChild( doc.createTextNode( m_description ) );

   // Now comes the keyword list
   QDomElement keylist = doc.createElement( "keywordlist" );
   root.appendChild( keylist );
   
   
   // Write the document to the buffer
   QByteArray array;
   QTextStream str( array, IO_WriteOnly );
   str.setEncoding( QTextStream::UnicodeUTF8 );
   str << doc.toString( );
   m_data->writeFile( "objectinfo.xml", "user", "group", array.size( ), array.data( ) );
}

void PMLibraryObject::savePreview( )
{
   QByteArray array;
   QDataStream iod( array, IO_WriteOnly );

   if( m_previewLoaded )
   {
      iod << *m_preview;
      m_data->writeFile( "preview.png", "user", "group", array.size( ), array.data( ) );
   }
}

void PMLibraryObject::saveObjects( )
{
   if( m_objectsLoaded )
      m_data->writeFile( "objectdata.kpm", "user", "group", m_objects->size( ), m_objects->data( ) );
}
